@@ -3,10 +3,10 @@ import re
 PORT_SPEC = re.compile(
     "^"  # Match full string
     "("  # External part
-    "((?P<host>[a-fA-F\d.:]+):)?"  # Address
-    "(?P<ext>[\d]*)(-(?P<ext_end>[\d]+))?:"  # External range
+    r"((?P<host>[a-fA-F\d.:]+):)?"  # Address
+    r"(?P<ext>[\d]*)(-(?P<ext_end>[\d]+))?:"  # External range
     ")?"
-    "(?P<int>[\d]+)(-(?P<int_end>[\d]+))?"  # Internal range
+    r"(?P<int>[\d]+)(-(?P<int_end>[\d]+))?"  # Internal range
     "(?P<proto>/(udp|tcp))?"  # Protocol
     "$"  # Match full string
 )
@@ -54,6 +54,12 @@ def port_range(start, end, proto, randomly_available_port=False):
 
 
 def split_port(port):
+    if hasattr(port, 'legacy_repr'):
+        # This is the worst hack, but it prevents a bug in Compose 1.14.0
+        # https://github.com/docker/docker-py/issues/1668
+        # TODO: remove once fixed in Compose stable
+        port = port.legacy_repr()
+    port = str(port)
     match = PORT_SPEC.match(port)
     if match is None:
         _raise_invalid_port(port)
